## Copyright (C) 2012 Lorant Oroszlany
##
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 3 of the License, or
## (at your option) any later version.
##
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with Octave; see the file COPYING.  If not, see
## <http://www.gnu.org/licenses/>.

## Mel-frequency cepstral coefficients
##  plp(X, sf, minf, maxf, ncoef, preemph) computes mel-frequency cepstral
##  coefficients from signal X.
##
##  ARGUMENTS (default):
##      sf (16000)  : sampling frequency
##      minf (0)    : lower bound of frequencies used for computation
##      maxf (8000) : upper bound of frequencies used for computation
##      ncoef       : number of triangular bandpass filters applied in
##                    bandwidth (maxf - minf)
##                    interval between filters defaults to 140 mel
##      preemph (5) : preemphasis coefficient

## Author: Lorant Oroszlany
## Created: 2012-04-26

function cm = mfcc (X,sf,minf,maxf,ncoef,preemph)

% set default options
if nargin < 2; sf = 16000; end;
if nargin < 3; minf = 0; end;
if nargin < 4; maxf = 8000; end;
if nargin < 5;
	ncoef = floor((hz2mel(maxf) - hz2mel(minf))/140);
end;
if nargin < 7; preemph = 0.97; end;


% apply pre-emphasis
if preemph ~= 0;
	X = filter([1 -preemph], 1, X);
end;

% compute constant step size on mel scale
Bmw = hz2mel(maxf) - hz2mel(minf); 	% bandwith in mels
delta = Bmw/(ncoef + 1); 			% step size

% central freqs in mels
bm = zeros(ncoef+2,1);
bm(1) = hz2mel(minf);
for i = 2:ncoef+2
	bm(i) = bm(i-1) + delta;
end;

% convert central frequencies back to Hz scale
b = mel2hz(bm);

% short-time fourier transform
winsize = floor(sf/40);
increment = floor(sf/100);
coef = ceil(sf/80);
Y = stft(X, winsize, increment, coef, 'hamming');

% map frequency to samples
x = round(b.*((coef*2)/sf));
if x(1) == 0
	x(1) = X(1) + 1;
end;

% filter output
nseg = size(Y)(2);
ym = zeros(ncoef,nseg);

for seg = 1:nseg;
	for i = 1:ncoef
			ym(i,seg) =  sum( Y(x(i):x(i+2),seg) .* bartlett(x(i+2)-x(i)+1) );
	end;
end;

ym = log10(ym);

% mel cepstral coefficients
cm = dct_n(ym);

endfunction
