## Copyright (C) 2012 Lorant Oroszlany
## 
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 3 of the License, or
## (at your option) any later version.
## 
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
## 
## You should have received a copy of the GNU General Public License
## along with Octave; see the file COPYING.  If not, see
## <http://www.gnu.org/licenses/>.

## Perceptual Linear Prediction
##  plp(X, sf, minf, maxf, M, Q) computes perceptual linear prediction
##  coefficients from signal X.
##
##  ARGUMENTS (default):
##      sf (16000)  : sampling frequency
##      minf (0)    : lower bound of frequencies used for computation
##      maxf (8000) : upper bound of frequencies used for computation
##      M (21)      : number of critical band filters applied in bandwidth 
##                    (maxf - minf)
##      Q (5)       : model order

## Author: Lorant Oroszlany
## Created: 2012-04-28

function [ ret ] = plp (X, sf, minf, maxf, M, Q)

if nargin < 2; sf = 16000; end;
if nargin < 3; minf = 0; end;
if nargin < 4; maxf = sf/2; end;
if nargin < 5; M = 21; end;
if nargin < 6; Q = 5; end;

S = fft(X.*hamming(length(X)))(1:ceil(length(X)/2));
P = abs(S).^2;		% power spectrum of X

% central frequencies in barks 		
Bbw = hz2bark(maxf) - hz2bark(minf);		% bandwidth in barks
delta = Bbw/(M - 1);

bb = zeros(M,3);	% [central frequency, low bw border, high bw border] of filter i
bb(1,1) = hz2bark(minf);

for i = 2:M
	bb(i,1) = bb(i-1,1) + delta;
	bb(i,2) = bb(i,1) - 2.5;
	bb(i,3) = bb(i,1) + 1.3;
end

b = bark2hz(bb);		% filter central & border frequencies in Hz

% map frequency to samples
x = round(b.*((length(P)*2)/sf));

E = elc(length(P),sf);
P = P.*E;		% pre-emphasis with equal loundness curve

for i = 1:M
	for j = 1:3
		if x(i,j) < 1; x(i,j) = 1; end;
		if x(i,j) > length(P); x(i,j) = length(P); end;
	end
end

XI = zeros(M,1);

for i = 2:M-1
	XI(i) = sum( P(x(i,2):x(i,3)) .* cbandfilt(x(i,3)-x(i,2)+ 1) ) ;
end

XI(1) = XI(2);
XI(M) = XI(M-1);

xi = XI .^ (0.3);

R = zeros(Q+1,1);

for i = 0:Q
	R(i+1) = 1/(2*M-1) * ( xi(1)*cos(i*0) + 2*( sum(xi(2:M-1) .* cos( b(2:M-1,1)*2*pi/sf*i ) ) ) + xi(M)*cos(i*pi) );
end

ret = levinson(R);

endfunction
